# $RCSfile: activeareaPage.py,v $
# $Revision: 1.14 $
# $Author: langer $
# $Date: 2004/12/22 03:05:40 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

# GUI page for manipulating pixel groups.

from SWIG.common import switchboard
##from common import activearea
from common import activeareamod
from common import debug
from common import guitop
##from common import labeltree
from common import microstructure
##from common import ringbuffer
#from common import pixelselection
#from common import pixelselectionmod
from common import utils
##from common.IO import whoville
from common.IO import questioner
from common.IO.GUI import chooser
from common.IO.GUI import historian
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import regclassfactory
from common.IO.GUI import whowidget
from common.IO import mainmenu
from gtk import *

switch = switchboard.switchboard

class ActiveAreaPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Active Area", ordering=71.1,
                                 tip="Modify active area.")

        mainbox = GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        # Microstructure widget, centered at the top of the page.
        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        label = GtkLabel('Microstructure=')
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        self.mswidget = whowidget.WhoWidget(microstructure.microStructures,
                                            scope=self)
        centerbox.add(self.mswidget.gtk[0], expand=0, fill=0)

        mainpane = GtkHPaned()
        mainbox.add(mainpane)

        # Active area status in the left half of the main pane.
        vbox = GtkVBox()
        mainpane.pack1(vbox, resize=1, shrink=0)
        aasframe = GtkFrame("Active Area Status")
        vbox.add(aasframe, expand=0, fill=0, padding=2)
        self.aainfo = GtkLabel()
        self.aainfo.set_alignment(0.0, 0.5)
        aasframe.add(self.aainfo)

        naaframe = GtkFrame("Named Active Areas")
        vbox.add(naaframe, expand=1, fill=1)
        naabox = GtkVBox()
        naaframe.add(naabox)
        self.aalist = chooser.ScrolledChooserListWidget(callback=self.aalistCB,
                                                      dbcallback=self.aalistCB2)
        naabox.add(self.aalist.gtk, padding=2)
        bbox = GtkHBox()
        naabox.add(bbox, expand=0, fill=0, padding=2)
        self.storebutton = GtkButton("Store...")
        bbox.add(self.storebutton, expand=1, fill=0)
        self.storebutton.connect('clicked', self.storeCB)
        guitop.top().tooltip(self.storebutton,
                             "Save the current active area for future use.")
        self.renamebutton = GtkButton("Rename...")
        bbox.add(self.renamebutton, expand=1, fill=0)
        self.renamebutton.connect('clicked', self.renameCB)
        guitop.top().tooltip(self.renamebutton,
                             "Rename the selected saved active area.")
        self.deletebutton = GtkButton("Delete")
        bbox.add(self.deletebutton, expand=1, fill=0)
        self.deletebutton.connect('clicked', self.deleteCB)
        guitop.top().tooltip(self.deletebutton,
                             "Delete the selected saved active area.")
        self.restorebutton = GtkButton("Restore")
        bbox.add(self.restorebutton, expand=1, fill=0)
        self.restorebutton.connect('clicked', self.restoreCB)
        guitop.top().tooltip(self.restorebutton,
                             "Use the selected saved active area.")
        

##        aasframe = GtkFrame("Active Area Status")
##        mainpane.pack1(aasframe, resize=1, shrink=0)
##        datascroll = GtkScrolledWindow()
##        aasframe.add(datascroll)
##        datascroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
##        self.aainfo = GtkText()
##        self.aainfo.set_editable(0)
##        datascroll.add(self.aainfo)


        # Active area modification methods in the right half of the main pane
        modframe = GtkFrame("Active Area Modification")
        mainpane.pack2(modframe, resize=1, shrink=0)
        modbox = GtkVBox()
        modframe.add(modbox)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        modbox.add(scroll)
        self.activeareaModFactory = regclassfactory.RegisteredClassFactory(
            activeareamod.ActiveAreaModifier.registry, title="Method:",
            scope=self)
        scroll.add_with_viewport(self.activeareaModFactory.gtk)
        self.historian = historian.Historian(self.activeareaModFactory.set,
                                             self.sensitizeHistory,
                                             setCBkwargs={'interactive':1})
        self.activeareaModFactory.set_callback(self.historian.stateChangeCB)

        # Prev, OK, and Next buttons
        hbox = GtkHBox()
        modbox.add(hbox, expand=0, fill=0, padding=2)
        self.prevbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkArrow(at=ARROW_LEFT))
        lbox.add(GtkLabel(' Prev'))
        self.prevbutton.add(lbox)
        hbox.add(self.prevbutton, expand=0, fill=0, padding=2)
        self.prevbutton.connect('clicked', self.historian.prevCB)
        guitop.top().tooltip(self.prevbutton,
                     'Recall the previous active area modification operation.')
        self.okbutton = GtkButton('OK')
        hbox.add(self.okbutton, expand=1, fill=1, padding=2)
        self.okbutton.connect('clicked', self.okbuttonCB)
        guitop.top().tooltip(self.okbutton,
              'Perform the active area modification operation defined above.')
        self.nextbutton = GtkButton()
        lbox = GtkHBox()
        lbox.add(GtkLabel('Next '), expand=0, fill=0)
        lbox.add(GtkArrow(at=ARROW_RIGHT), expand=0,fill=0)
        self.nextbutton.add(lbox)
        hbox.add(self.nextbutton, expand=0, fill=0, padding=2)
        self.nextbutton.connect('clicked', self.historian.nextCB)
        guitop.top().tooltip(self.nextbutton,
                        "Recall the next active area modification operation.")

        # Undo, Redo, Override
        hbox = GtkHBox()
        modbox.add(hbox, expand=0, fill=0, padding=2)
        self.undobutton = GtkButton('Undo')
        hbox.add(self.undobutton, expand=1, fill=0)
        self.undobutton.connect('clicked', self.undoCB)
        guitop.top().tooltip(self.undobutton, "Undo the previous operation.")

        self.redobutton = GtkButton('Redo')
        hbox.add(self.redobutton, expand=1, fill=0)
        self.redobutton.connect('clicked', self.redoCB)
        guitop.top().tooltip(self.redobutton, "Redo an undone operation.")

        self.overridebutton = GtkToggleButton('Override')
        hbox.add(self.overridebutton, expand=1, fill=0)
        self.overridesignal = self.overridebutton.connect('clicked',
                                                          self.overrideCB)
        guitop.top().tooltip(self.overridebutton,
                             "Temporarily activate the entire microstructure.")

        # Switchboard signals
        self.sbcallbacks = [
            switchboard.requestCallback(self.mswidget, self.mswidgetCB),
            switchboard.requestCallback("active area modified",
                                        self.aamodified),
            switchboard.requestCallback("stored active areas changed",
                                        self.storedAAChanged),
            switchboard.requestCallback(('validity', self.activeareaModFactory),
                                        self.validityChangeCB)
        ]

        # Sensitizing
        self.sensitize()
        self.sensitizeHistory()
        
    def getCurrentMSName(self):
        return self.mswidget.get_value()
    def getCurrentMS(self):
        try:
            return microstructure.microStructures[self.getCurrentMSName()]
        except KeyError:
            pass
    def getCurrentActiveArea(self):
        return self.aalist.get_value()

    # Callback functions for active area modification

    def okbuttonCB(self, button):
        modmeth = self.activeareaModFactory.getRegistration()
        if modmeth is not None:
            self.activeareaModFactory.set_defaults()
            menuitem = getattr(mainmenu.OOF.ActiveArea,
                               utils.space2underscore(modmeth.name()))
            menuitem.callWithDefaults(microstructure=self.getCurrentMSName())

    def undoCB(self, button):
        mainmenu.OOF.ActiveArea.Undo.callWithDefaults(
            microstructure=self.getCurrentMSName())

    def redoCB(self, button):
        mainmenu.OOF.ActiveArea.Redo.callWithDefaults(
            microstructure=self.getCurrentMSName())

    def overrideCB(self, button):
        mainmenu.OOF.ActiveArea.Override(microstructure=self.getCurrentMSName(),
                                override=button.get_active())
        
    def mswidgetCB(self, *args, **kwargs): # switchboard self.mswidget
        self.updateAAInfo()
        self.updateAAList()
        self.sensitize()
        self.setOverrideButton()

    def aalistCB(self, name):           # activearealist callback
        self.sensitize()

    def aalistCB2(self, name):          # activearealist double-click callback
        self.sensitize()
        self.restoreCB(None)
        
    def storeCB(self, button):
        menuitem = mainmenu.OOF.ActiveArea.Store
        if parameterwidgets.getParameters(menuitem.get_arg('name'),
                                          title="Store the active area"):
            menuitem.callWithDefaults(microstructure=self.getCurrentMSName())

    def renameCB(self, button):
        menuitem = mainmenu.OOF.ActiveArea.Rename
        namearg = menuitem.get_arg('newname')
        namearg.value = self.getCurrentActiveArea()
        oldname = self.getCurrentActiveArea()
        if parameterwidgets.getParameters(menuitem.get_arg('newname'),
                                          title="Rename activearea '%s'"
                                               % oldname):
            menuitem.callWithDefaults(microstructure=self.getCurrentMSName(),
                                      oldname=oldname)

    def deleteCB(self, button):
        if questioner.questioner("Really delete %s?" %
                                 self.getCurrentActiveArea(),
                                 "No", default="Yes") == "Yes":
            mainmenu.OOF.ActiveArea.Delete(microstructure=self.getCurrentMSName(),
                                  name=self.getCurrentActiveArea())

    def restoreCB(self, button):
        mainmenu.OOF.ActiveArea.Restore(microstructure=self.getCurrentMSName(),
                               name=self.getCurrentActiveArea())
        

    def aamodified(self, modifier):
        # callback for switchboard signal "active area modified"
        self.updateAAInfo()
        self.updateHistory(modifier)
        self.sensitize()
        self.setOverrideButton()

    def storedAAChanged(self, name):
        # callback for switchboard signal "stored active areas changed"
        self.updateAAList()
        try:
            self.aalist.set_selection(name)
        except KeyError:
            pass
        self.sensitize()

    def updateAAList(self):             # update list of named active areas
        ms = self.getCurrentMS()
        if ms is not None:
            self.aalist.update(ms.getObject().activeAreaNames())
        else:
            self.aalist.update([])

    def updateAAInfo(self):
        ms = self.getCurrentMS()
        if ms is not None:
            activearea = ms.getObject().activearea
            aasize = activearea.size()
            mssize = ms.getObject().sizeInPixels()
            mpxls = mssize[0] * mssize[1]
            apxls = mpxls - aasize
            msg = "%d of %d pixels are active" % (apxls, mpxls)
            self.aainfo.set_text(msg)
        else:
            self.aainfo.set_text("No Microstructure selected")

    def setOverrideButton(self):
        ms = self.getCurrentMS()
        if ms is not None:
            self.overridebutton.signal_handler_block(self.overridesignal)
            self.overridebutton.set_active(
                ms.getObject().activearea.getOverride())
            self.overridebutton.signal_handler_unblock(self.overridesignal)

    def updateHistory(self, activeareaModifier):
        if activeareaModifier is not None:
            self.historian.record(activeareaModifier)
            
    def sensitize(self):
        debug.mainthreadTest()
        ms = self.getCurrentMS()
        if ms is not None:
            activearea = ms.getObject().activearea
            self.undobutton.set_sensitive(activearea.undoable())
            self.redobutton.set_sensitive(activearea.redoable())
            self.overridebutton.set_sensitive(1)
            self.okbutton.set_sensitive(self.activeareaModFactory.isValid())
            self.storebutton.set_sensitive(1)
            aachosen = self.aalist.get_value() is not None
            self.restorebutton.set_sensitive(aachosen)
            self.deletebutton.set_sensitive(aachosen)
            self.renamebutton.set_sensitive(aachosen)
        else:
            self.undobutton.set_sensitive(0)
            self.redobutton.set_sensitive(0)
            self.overridebutton.set_sensitive(0)
            self.okbutton.set_sensitive(0)
            self.storebutton.set_sensitive(0)
            self.restorebutton.set_sensitive(0)
            self.deletebutton.set_sensitive(0)
            self.renamebutton.set_sensitive(0)

    def sensitizeHistory(self):
        debug.mainthreadTest()
        self.nextbutton.set_sensitive(self.historian.nextSensitive())
        self.prevbutton.set_sensitive(self.historian.prevSensitive())

    def validityChangeCB(self, validity):
        self.sensitize()

####################################
        
aap = ActiveAreaPage()
