# -*- python -*-
# $RCSfile: questioner.py,v $
# $Revision: 1.17 $
# $Author: langer $
# $Date: 2003/11/21 21:36:52 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 


# The function 'questioner' brings up a dialog box containing a
# question and a bunch of buttons corresponding to the possible
# answers.  The answer clicked on is returned.  The arguments are the
# question, followed by the answers.  An optional keyword argument,
# 'default', specifies which answer has the focus when the dialog is
# brought up.  If the value of the default argument isn't in the list
# of answers, it is *prepended* to the list.

import gtk, threading
from common import debug
from common import guitop
from common import mainthread
from common.IO.GUI import oof_mainiteration
from common import thread_enable
from SWIG.common import ooferror

class _Questioner:
    def __init__(self, question, *answers, **kwargs):
        debug.mainthreadTest()

        if len(answers)==0:
            raise ooferror.ErrSetupError(
                "Questioner must have at least one possible answer.")

        self.answers = answers
        self.gtk = gtk.GtkDialog()
        self.gtk.set_transient_for(guitop.top().gtk)
        hbox = gtk.GtkHBox()
        self.destroyCBID = self.gtk.connect("destroy", self.destroyCB)
        self.gtk.vbox.add(hbox, padding=15)
        hbox.add(gtk.GtkLabel(question), padding=15)
        self.defaultbutton = None

        try:
            self.default = kwargs['default']
        except KeyError:
            self.default=None
        else:
            if not self.default in answers:
                self.answers = (self.default,)+answers

        for answer in self.answers:
            button = gtk.GtkButton(answer)
            self.gtk.action_area.add(button)
            button.connect('clicked', self.buttonCB, answer)
            if answer == self.default:
                self.defaultbutton = button
        self.gtk.set_modal(1)
        self.answer = None
    def buttonCB(self, button, answer):
        self.answer = answer

    # In case of surprise destruction, return the default answer
    # if there is one, otherwise return the first answer in the list.
    def destroyCB(self, gtkobj):
        if self.default:
            self.answer = self.default
        else:
            self.answer = self.answers[0]
        self.gtk = None
    def inquire(self):
        debug.mainthreadTest()
        self.answer = None
        self.gtk.show_all()
        if self.defaultbutton:
            self.defaultbutton.grab_focus()
        while self.answer is None:
            oof_mainiteration.mainiteration()
        if self.gtk:
            self.gtk.disconnect(self.destroyCBID)
            self.gtk.destroy()
        return self.answer


# Wrapper, callable from any thread, which calls questionerGUI
# on the main thread, blocking until it gets the result.
def questioner_(question, *args, **kwargs):
    q = mainthread.runBlock(_Questioner, (question,)+args, kwargs)
    result = mainthread.runBlock(q.inquire)
    return result


questioner = questioner_

# Override non-GUI questioner.
import common.IO.questioner
common.IO.questioner.questioner = questioner_
