# -*- python -*-
# $RCSfile: viewertoolboxGUI.py,v $
# $Revision: 1.8 $
# $Author: haan $
# $Date: 2004/12/28 22:30:26 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common import guitop
from common import mainthread
from common import primitives
from common.IO import reporter
from common.IO import viewertoolbox
from common.IO.mainmenu import OOF
from common.IO.GUI import mousehandler
from common.IO.GUI import toolboxGUI
from gtk import *

class ViewerToolboxGUI(toolboxGUI.GfxToolbox, mousehandler.MouseHandler):
    def __init__(self, viewertoolbox):
        debug.mainthreadTest()

        toolboxGUI.GfxToolbox.__init__(self, "Viewer", viewertoolbox)
        mainbox = GtkVBox()
        self.gtk.add(mainbox)

        infoframe = GtkFrame("Position Information")
        infotable = GtkTable(cols=3, rows=3)
        pixellabel = GtkLabel("Pixel: ")
        self.pixel_x = GtkEntry()
        self.pixel_x.set_usize(25, -1)
        self.pixel_x.set_editable(0)
        self.pixel_y = GtkEntry()
        self.pixel_y.set_usize(25, -1)
        self.pixel_y.set_editable(0)
        physicallabel = GtkLabel("Physical: ")
        self.physical_x = GtkEntry()
        self.physical_x.set_usize(80, -1)
        self.physical_x.set_editable(0)
        self.physical_y = GtkEntry()
        self.physical_y.set_usize(80, -1)
        self.physical_y.set_editable(0)
        canvaslabel = GtkLabel("Canvas: ")
        self.canvas_x = GtkEntry()
        self.canvas_x.set_usize(80, -1)
        self.canvas_x.set_editable(0)
        self.canvas_y = GtkEntry()
        self.canvas_y.set_usize(80, -1)
        self.canvas_y.set_editable(0)        
        infotable.attach(pixellabel, 0,1, 0,1, xpadding=5, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.pixel_x, 1,2, 0,1, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.pixel_y, 2,3, 0,1, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(physicallabel, 0,1, 1,2, xpadding=5, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.physical_x, 1,2, 1,2, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.physical_y, 2,3, 1,2, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(canvaslabel, 0,1, 2,3, xpadding=5, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.canvas_x, 1,2, 2,3, xpadding=2, ypadding=2,
                         xoptions=FILL)
        infotable.attach(self.canvas_y, 2,3, 2,3, xpadding=2, ypadding=2,
                         xoptions=FILL)        
        infoframe.add(infotable)
        mainbox.add(infoframe, fill=0, expand=0)

        zoomframe = GtkFrame("Zoom")
        zoombox = GtkVBox()
        settingtable = GtkTable(cols=2, rows=1)
        zfactorlabel = GtkLabel("Factor: ")
        self.zoomfactor = GtkEntry()
        self.zoomfactor.set_editable(1)
        self.zoomfactor.set_usize(137, -1)
        settingtable.attach(zfactorlabel, 0,1, 0,1, xpadding=5, ypadding=3,
                            xoptions=FILL)
        settingtable.attach(self.zoomfactor, 1,2, 0,1, xpadding=5, ypadding=3,
                            xoptions=FILL)

        self.mode = "Focussed"  # Initial setting for zoom mode
        zoommodebox = GtkHBox()
        zoommodebox.add(GtkLabel("Mode: "), expand=0, fill=0, padding=5)
        zoomModes = ["Focussed", "Normal"]
        self.modebuttons = []
        self.modebuttondict = {}
        for mode in zoomModes:
            if self.modebuttons:
                button = GtkRadioButton(label=mode,
                                        group=self.modebuttons[0])
            else:
                button = GtkRadioButton(label=mode)
            self.modebuttons.append(button)
            self.modebuttondict[mode] = button
            zoommodebox.add(button, expand=0, fill=0)
            button.set_active(self.mode == mode)
            button.connect('clicked', self.changeMode, mode)
        guitop.top().tooltip(self.modebuttondict["Normal"], "Zooms in with the center of the canvas focused.")
        guitop.top().tooltip(self.modebuttondict["Focussed"], "Zooms with the mouse pointer focused.")

        label0 = GtkLabel("Click: Zoom in\nShift+Click: Zoom out\nCtrl+Click: Fill window")
        label0.set_pattern("       _______\n             ________\n            ___________")
        label0.set_justify(JUSTIFY_LEFT)
        
        zoombox.add(settingtable, fill=0, expand=0)
        zoombox.add(zoommodebox, fill=0, expand=0)
        zoombox.add(GtkHSeparator(), fill=0, expand=0, padding=2)
        zoombox.add(label0, fill=0, expand=0)
        zoomframe.add(zoombox)
        mainbox.add(zoomframe, fill=0, expand=0)

        infobox = GtkHButtonBox()
        canvas_info = GtkButton("Canvas Info")
        canvas_info.connect("clicked", self.canvas_infoCB)
        guitop.top().tooltip(canvas_info,
                           "Display canvas information in the message window.")
        infobox.add(canvas_info)
        mainbox.add(infobox)

        self.currentZFactor = self.gfxwindow().zoomFactor()

        switchboard.requestCallback("zoom factor changed", self.zfactorCB)

    def activate(self):
        toolboxGUI.GfxToolbox.activate(self)
        self.gfxwindow().setMouseHandler(self)
        self.updateZFactor()

    def zfactorCB(self, *args):
        self.updateZFactor()
        
    def updateZFactor(self):
        self.currentZFactor = self.gfxwindow().zoomFactor()
        self.zoomfactor.set_text("%g"% self.currentZFactor)

    def changeMode(self, button, mode):
        self.mode = mode

    def findMicrostructure(self):
        who = self.toolbox.gfxwindow().topwho('Microstructure', 'Image',
                                              'Skeleton', 'Mesh')
        if who is not None:
            return who.getMicrostructure()

    def acceptEvent(self, eventtype):
        return eventtype in ('down', 'move', 'up')

    # World coord from world_coord() needs to be recalibrated.
    def recalibrate(self, point):
        return primitives.Point(point.x, -point.y)

    def get_proper_world_coord(self, i, j):
        temp = self.gfxwindow().oofcanvas.world_coord(i, j)
        return self.recalibrate(temp)

    def canvas_infoCB(self, *args):
        # Visible canvas size in pixel units
        xmax = self.gfxwindow().oofcanvas.get_width_in_pixels()-1
        ymax = self.gfxwindow().oofcanvas.get_height_in_pixels()-1
        # Visible MS in physical units
        ll = self.get_proper_world_coord(0, ymax)
        ur = self.get_proper_world_coord(xmax, 0)
        reporter.report("### Canvas Information ###")
        reporter.report("Lower-left corner : ", ll)
        reporter.report("Upper-right corner: ", ur)
        reporter.report("H-scroll position : ",
                        self.gfxwindow().oofcanvas.hScrollPosition())
        reporter.report("V-scroll position : ",
                        self.gfxwindow().oofcanvas.vScrollPosition())
        reporter.report("Pixels per unit   : ",
                        self.gfxwindow().oofcanvas.get_pixels_per_unit())

    # Mouse position in the canvas, "y" is inverted.
    def get_canvas_position(self, x, y):
        # Visible canvas size in pixel units
        xmax = self.gfxwindow().oofcanvas.get_width_in_pixels()-1
        ymax = self.gfxwindow().oofcanvas.get_height_in_pixels()-1
        # Visible MS in physical units
        ul = self.get_proper_world_coord(0, 0)
        lr = self.get_proper_world_coord(xmax, ymax)
        # Calculates click point's position in the canvas
        click_x = (x-ul.x)*xmax/(lr.x-ul.x)
        click_y = (y-ul.y)*ymax/(lr.y-ul.y)
        return primitives.iPoint(click_x, click_y)

    # What to do with mouse?
    # 1. Click: Zoom in
    # 2. shift+Click: Zoom out
    # 3. Ctrl+Click: Fill windows
    def down(self, x, y, shift, ctrl):
        debug.mainthreadTest()
        pass

    def up(self, x, y, shift, ctrl):
        debug.mainthreadTest()
##        if self.mode == "Focussed":
##            # Need to know where I am in the canvas at the time of clicking.
##            # The origin of the canvas is in the upper-left corner!
##            canvas_point = self.get_canvas_position(x, y)

        if not shift and not ctrl:
            try:
                factor = float(self.zoomfactor.get_text())
                if factor != self.currentZFactor:
                    self.gfxwindow().menu.Settings.Zoom.Zoom_Factor(
                        factor=factor)
            except ValueError:
                reporter.report("Invalid value for Zoom Factor.")
            else:
                if self.mode == "Focussed":
                    self.gfxwindow().menu.Settings.Zoom.InFocussed(
                        focus=primitives.Point(x,y))
                else:
                    self.gfxwindow().menu.Settings.Zoom.In()
        elif shift and not ctrl:
            try:
                factor = float(self.zoomfactor.get_text())
                if factor != self.currentZFactor:
                    self.gfxwindow().menu.Settings.Zoom.Zoom_Factor(
                        factor=factor)
            except ValueError:
                reporter.report("Invalid value for Zoom Factor.")
            else:
                if self.mode == "Focussed":
                    self.gfxwindow().menu.Settings.Zoom.OutFocussed(
                        focus=primitives.Point(x,y))
                else:
                    self.gfxwindow().menu.Settings.Zoom.Out()

        elif not shift and ctrl:
            self.gfxwindow().menu.Settings.Zoom.Fill_Window()
        else:
            reporter.report("Seems that you're holding both shift and ctrl.")

##        if self.mode == "Focussed" and not (not shift and ctrl):
##            # Where am I now after the zoom.
##            now = self.get_proper_world_coord(canvas_point.x,
##                                              canvas_point.y)
##            ppu = self.gfxwindow().oofcanvas.get_pixels_per_unit()
##            diff_x = (now.x - x)*ppu
##            diff_y = (now.y - y)*ppu
##            # Need to get the scroll position
##            hs_pos = self.gfxwindow().oofcanvas.hScrollPosition()
##            vs_pos = self.gfxwindow().oofcanvas.vScrollPosition()
##            # Now, it's time to move
##            self.gfxwindow().oofcanvas.hScrollTo(hs_pos - diff_x)
##            self.gfxwindow().oofcanvas.vScrollTo(vs_pos + diff_y)

##    def move(self, x, y, shift, ctrl):
##        debug.mainthreadTest()
##        ms = self.findMicrostructure()
##        if ms:
##            pixel = ms.pixelFromPoint(primitives.Point(x,y))
##            self.pixel_x.set_text("%d" % pixel.x)
##            self.pixel_y.set_text("%d" % pixel.y)
##            self.physical_x.set_text("%-11.4g" % x)
##            self.physical_y.set_text("%-11.4g" % y)
##            canvas_position = self.get_canvas_position(x, y)
##            self.canvas_x.set_text("%d" % canvas_position.x)
##            self.canvas_y.set_text("%d" % canvas_position.y)

def _makeGUI(self):
    return ViewerToolboxGUI(self)

viewertoolbox.ViewerToolbox.makeGUI = _makeGUI
    
