# -*- python -*-
# $RCSfile: activearea_.py,v $
# $Revision: 1.1 $
# $Author: langer $
# $Date: 2004/05/07 19:48:05 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

## The active regions of a Microstructure are stored in an ActiveArea
## object, which is basically the same as a PixelSelection.
## ActiveAreas and PixelSelections share a C++ base class,
## CPixelSelection.  Each Python Microstructure holds a Python
## ActiveAreaContext, which is the WhoDoUndoClass for ActiveAreas.
## The CMicrostructures contain a pointer to the current ActiveArea in
## the ActiveAreaContext.

## In the CPixelSelection object, it's the INactive pixels that are
## explicitly stored, so that clearing the ActiveArea is the same as
## making all pixels active.

from SWIG.common import switchboard
from common import color
from common import debug
from common.IO import whoville
from common.IO import display
from common.IO import bitoverlaydisplay
from common.IO import topwho            # defines <top activearea>

class NamedActiveArea:
    def __init__(self, name, activearea):
        self.name = name
        self.activearea = activearea

#####################################

class ActiveAreaContext(whoville.WhoDoUndo):
    def __init__(self, path, classname, obj):
        # override==1 means that all pixels are active, despite what's
        # stored in the pixel selection.
        self._override = 0
        whoville.WhoDoUndo.__init__(self, path, classname, obj)
    def start(self):
        newselection = self.getObject().clone()
        self.pushModification(newselection)
    def selectWithoutCheck(self, selectioncourier):
        self.getObject().selectWithoutCheck(selectioncourier)
        switchboard.notify('pixel selection changed')
    def unselectWithoutCheck(self, selectioncourier):
        self.getObject().unselectWithoutCheck(selectioncourier)
        switchboard.notify('pixel selection changed')
    def toggleWithoutCheck(self, selectioncourier):
        self.getObject().toggleWithoutCheck(selectioncourier)
        switchboard.notify('pixel selection changed')
    def clear(self):
        self.getObject().clearWithoutCheck()
    def invert(self):
        self.getObject().invertWithoutCheck()
    def undo(self):
        self.undoModification()
    def redo(self):
        self.redoModification()
    def clearable(self):
        return self.size() != 0
    def size(self):
        return len(self.getObject())
    def override(self, flag):
        # Keep the C++ flags in the ActiveAreas synchronized
        # with the Python flag in the ActiveAreaContext.
        for aa in self.undobuffer:
            aa.override(flag)
        self.getTimeStamp(None).increment()
        self._override = flag
    def getOverride(self):
        return self._override
    def isActive(self, pixel):
        return self.getObject().isActive(pixel)
    def getBitmap(self):
        if not self._override:
            return self.getObject().getBitmap()
    def getSelection(self):
        return self.getObject().members()
    def getSelectionAsGroup(self):
        return self.getObject().getPixelGroup()

activeareaWhoClass = whoville.WhoDoUndoClass(
    'Active Area',
    instanceClass=ActiveAreaContext,
    ordering=998,
    secret=1,
    proxyClasses=['<top activearea>'])

display.predefineLayer(
    'Active Area',
    '<top activearea>',
    bitoverlaydisplay.bitmapOverlay(color=color.white, opacity=0.8))


