# -*- python -*-
# $RCSfile: fieldPage.py,v $
# $Revision: 1.26 $
# $Author: redwing $
# $Date: 2004/12/17 18:03:48 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common import debug
from common import guitop
from common import labeltree
from common import microstructure
from common.IO import mainmenu
from common.IO import parameter
from common.IO.GUI import oofGUI
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import fieldinit
from engine import skeletoncontext
from gtk import *
import engine.mesh
import SWIG.engine.field
import string

allFields = SWIG.engine.field.allFields

meshmenu = mainmenu.OOF.Mesh

class ButtonSignal:
    def __init__(self, button, signal):
        self.button = button
        self.signal = signal
    def set(self, active):
        debug.mainthreadTest()
        self.button.signal_handler_block(self.signal)
        self.button.set_active(active)
        self.button.signal_handler_unblock(self.signal)
    def set_sensitive(self, sensitivity):
        debug.mainthreadTest()
        self.button.set_sensitive(sensitivity)


class FieldPage(oofGUI.MainPage):
    def __init__(self):
        oofGUI.MainPage.__init__(self, name="Fields", ordering=210,
                                 tip="Define fields on a finite element mesh.")
        mainbox = GtkVBox(spacing=2)
        self.gtk.add(mainbox)

        hbox = GtkHBox()
        mainbox.add(hbox, expand=0, fill=0)
        centerbox = GtkHBox(spacing=3)
        hbox.add(centerbox, expand=1, fill=0)
        self.meshwidget = whowidget.WhoWidget(engine.mesh.meshes,
                                              callback=self.meshCB, scope=self)
        label = GtkLabel("Microstructure=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[0], expand=0, fill=0)
        label = GtkLabel("Skeleton=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[1], expand=0, fill=0)
        label = GtkLabel("Mesh=")
        label.set_alignment(1.0, 0.5)
        centerbox.add(label, expand=0, fill=0)
        centerbox.add(self.meshwidget.gtk[2], expand=0, fill=0)

        vpane = GtkVPaned()
        mainbox.add(vpane, padding=3)
        vpane.set_gutter_size(10)

        fieldframe = GtkFrame("Fields")
        vpane.pack1(fieldframe, resize=1, shrink=0)
        scroll = GtkScrolledWindow()
        scroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        fieldframe.add(scroll)
        bbox = GtkVBox()
        scroll.add_with_viewport(bbox)
        self.fieldtable = GtkTable()
        bbox.add(self.fieldtable, expand=0, fill=0)
        self.build_fieldTable()

        frame = GtkFrame("Field Initialization")
        vpane.pack2(frame, resize=1, shrink=0)
        ivbox = GtkVBox()
        frame.add(ivbox)
        self.initscroll = GtkScrolledWindow()
        self.initscroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
        ivbox.add(self.initscroll, expand=1, fill=1)
        self.initlist = GtkCList(cols=2, titles=['Field', 'Initializer'])
        self.initscroll.add(self.initlist)
        self.initlist.column_titles_passive()
        self.initlist.set_column_auto_resize(0, 1)
        self.initlist.set_column_auto_resize(1, 1)
        self.initlist.connect('select-row', self.initSelectCB)
        self.initlist.connect('unselect-row', self.initUnselectCB)
        self.initlist.connect('button_press_event', self.buttonpressCB)
        self.currentInit = None
        self.initmsg = GtkLabel("Fields defined above\ncan be initialized here\nafter a Mesh is created.")
        ivbox.add(self.initmsg, expand=1, fill=1)

        self.doubleclicked = 0
        # This tooltip doesn't appear to appear.
##         guitop.top().tooltips.set_tip(self.initlist,
##                                       'Select a field and click the "Set Initializer" button or double-click to set the field\'s initial values.  Only those fields defined above appear in this list.')
        
        bbox = GtkHButtonBox()
        ivbox.add(bbox, expand=0, fill=0)

        self.initbutton = GtkButton('Set Initializer...')
        self.initbutton.connect('clicked', self.initbuttonCB)
        guitop.top().tooltips.set_tip(self.initbutton,
                                      'Click to initialize the selected field. Alternatively, a double-click on the desired field brings up initializer.')
        bbox.add(self.initbutton)

        self.copybutton = GtkButton("Copy Field State...")
        self.copybutton.connect('clicked', self.copyfstateCB)
        bbox.add(self.copybutton)
        guitop.top().tooltips.set_tip(self.copybutton,
                                      "Copy all field status variables from the current mesh to another mesh.")

        self.sensitize()

        switchboard.requestCallback(("new who", "Skeleton"), self.newSkeleton)
        switchboard.requestCallback(("new who", "Mesh"), self.newMesh)
        switchboard.requestCallback(("remove who", "Mesh"), self.removeMesh)
        switchboard.requestCallback("made reservation", self.reservationChanged)
        switchboard.requestCallback("cancelled reservation",
                                    self.reservationChanged)
        switchboard.requestCallback("new field", self.newFieldCB)
        switchboard.requestCallback("field defined", self.defineFldCB)
        switchboard.requestCallback("field activated", self.activateFldCB)
        switchboard.requestCallback("field inplane", self.inplaneFldCB)
        switchboard.requestCallback("field initialized", self.initFldCB)

    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()
        self.update()

    def build_fieldTable(self):
        debug.mainthreadTest()
        self.fieldtable.foreach(GtkObject.destroy) # clear the table
        self.fieldbuttons = {}
        self.fieldtable.resize(len(allFields), 6)
        self.fieldtable.attach(GtkVSeparator(), 1,2, 0,len(allFields),
                               xoptions=0, yoptions=EXPAND|FILL)
        self.fieldtable.set_col_spacing(0, 3)
        row = 0
        for fld in allFields:
            label = GtkLabel(fld.name())
            label.set_alignment(1.0, 0.5)
            self.fieldtable.attach(label, 0,1, row,row+1,
                                   xoptions=FILL, yoptions=0)
            button = GtkCheckButton('defined')
            signal = button.connect('clicked', self.fieldDefineCB, fld)
            self.fieldbuttons[(fld.name(), "defined")] = ButtonSignal(button,
                                                                        signal)
            self.fieldtable.attach(button, 2,3, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldDefineTip(button, fld)

            button = GtkCheckButton('active')
            signal = button.connect('clicked', self.fieldActiveCB, fld)
            self.fieldbuttons[(fld.name(), "active")] = ButtonSignal(button,
                                                                     signal)
            self.fieldtable.attach(button, 3,4, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldActiveTip(button, fld)

            button = GtkCheckButton('in-plane')
            signal = button.connect('clicked', self.fieldInPlaneCB, fld)
            self.fieldbuttons[(fld.name(), "inplane")] = ButtonSignal(button,
                                                                    signal)
            self.fieldtable.attach(button, 4,5, row, row+1,
                                   xoptions=0, yoptions=0)
            self.setFieldInPlaneTip(button, fld)

            row += 1

    def newFieldCB(self):               # switchboard "new field"
        self.build_fieldTable()
        self.show()

    def fieldDefineCB(self, button, field): # gtk callback
        if button.get_active():
            meshmenu.Field.Define(mesh=self.currentFullMeshName(),
                                  field=field)
        else:
            meshmenu.Field.Undefine(mesh=self.currentFullMeshName(),
                                    field=field)
        self.setFieldDefineTip(button, field)
        self.sensitize() # other field actions require the field to be defined

    def setFieldDefineTip(self, button, field):
        if button.get_active():
            verb = "Undefine"
        else:
            verb = "Define"
        guitop.top().tooltip(button, "%s the %s field on the mesh.  Only defined fields have values." % (verb, field.name()))

    def fieldActiveCB(self, button, field): # gtk callback
        if button.get_active():
            meshmenu.Field.Activate(mesh=self.currentFullMeshName(),
                                    field=field)
            ## switchboard.notify("symmetry check", self.currentMSName(), self.currentFullMeshName())
        else:
            meshmenu.Field.Deactivate(mesh=self.currentFullMeshName(),
                                      field=field)
            ## switchboard.notify("symmetry check", self.currentMSName(), self.currentFullMeshName())
        self.setFieldActiveTip(button, field)
        
    def setFieldActiveTip(self, button, field):
        if button.get_active():
            verb = "Deactivate"
        else:
            verb = "Activate"
        guitop.top().tooltip(button, "%s the %s field on the mesh. The solver finds the values of the active fields by solving the active equations." % (verb, field.name()))

    def fieldInPlaneCB(self, button, field): # gtk callback
        if button.get_active():
            meshmenu.Field.In_Plane(mesh=self.currentFullMeshName(),
                                    field=field)
        else:
            meshmenu.Field.Out_of_Plane(mesh=self.currentFullMeshName(),
                                        field=field)
        self.setFieldInPlaneTip(button, field)
        
    def setFieldInPlaneTip(self, button, field):
        if button.get_active():
            verb = "Do not constrain"
        else:
            verb = "Constrain"
        guitop.top().tooltip(button, "%s the %s field to lie in the x-y plane."
                             % (verb, field.name()))

    def defineFldCB(self, meshname, fieldname, defined): # sb "field defined"
        if meshname == self.currentFullMeshName():
            self.fieldbuttons[(fieldname, "defined")].set(defined)
            self.update()

    def activateFldCB(self, meshname, fieldname, active): # sb "field activated"
        if meshname == self.currentFullMeshName():
            self.fieldbuttons[(fieldname, "active")].set(active)

    def inplaneFldCB(self, meshname, fieldname, inplane): # sb "field inplane"
        if meshname == self.currentFullMeshName():
            self.fieldbuttons[(fieldname, "inplane")].set(inplane)

    ########################

    def currentMSName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[0]
    def currentMS(self):
        msname = self.currentMSName()
        if msname:
            return microstructure.microStructures[msname].getObject()

    def currentSkeletonName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[1]
    def currentSkeleton(self):
        ms = self.currentMS()
        meshname = self.currentSkeletonName()
        if meshname:
            return skeletoncontext.getSkeleton(ms, meshname)

    def currentFullMeshName(self):
        return self.meshwidget.get_value()
    def currentMeshName(self):
        path = labeltree.makePath(self.currentFullMeshName())
        return path[2]
    def currentMeshContext(self):
        try:
            return engine.mesh.meshes[self.currentFullMeshName()]
        except KeyError:
            return None
    def currentMesh(self):
        try:
            return engine.mesh.meshes[self.currentFullMeshName()].getObject()
        except KeyError:
            return None
            
    def sensitize(self):
        debug.mainthreadTest()
        msok = self.currentMS() is not None
        skelok = msok and self.currentSkeleton() is not None
        meshok = skelok and self.currentMesh() is not None and \
                 not self.currentMeshContext().query_reservation()

        self.fieldtable.set_sensitive(meshok)
        self.initlist.set_sensitive(meshok)
        self.initbutton.set_sensitive(meshok and self.currentInit is not None)

        self.copybutton.set_sensitive(meshok)
        
    def update(self):
        self.set_state(self.currentFullMeshName())

    def set_state(self, meshpath):
        debug.mainthreadTest()
        path = labeltree.makePath(meshpath)
        self.meshwidget.set_value(path)
        if path[-1]:                    # ie, mesh name is not empty
            themesh = engine.mesh.meshes[path] # Who object
            femesh = themesh.getObject() # FEMesh object
            skel = themesh.skeleton
            self.initlist.clear()
            for fld in allFields:
                fname = fld.name()
                fdef = femesh.is_defined_field(fld)
                self.fieldbuttons[(fname, "defined")].set(fdef)
                self.fieldbuttons[(fname, "active")].set_sensitive(fdef)
                self.fieldbuttons[(fname, "inplane")].set_sensitive(fdef)
                if fdef:
                    self.fieldbuttons[(fname, "active")].set(
                        femesh.is_active_field(fld))
                    self.fieldbuttons[(fname, "inplane")].set(
                        femesh.in_plane(fld))
##                    init = themesh.get_initializer(fld)
##                    if init:
##                        self.initlist.append([fname+':', `init`])
##                    else:
##                        self.initlist.append([fname+':', '<uninitialized>'])
                else:                   # field not defined
                    self.fieldbuttons[(fname, "active")].set(0)
                    self.fieldbuttons[(fname, "inplane")].set(0)
            self.updateInitializers()
            self.initmsg.hide()
            self.initscroll.show_all()

        else:                           # no current mesh
            self.initscroll.hide()
            self.initmsg.show()
            for button in self.fieldbuttons.values():
                button.set(0)

    def updateInitializers(self):
        self.initlist.clear()
        mesh = self.currentMeshContext()
        if mesh:
            femesh = mesh.getObject()
            for fld in allFields:
                if femesh.is_defined_field(fld):
                    init = mesh.get_initializer(fld)
                    if init:
                        self.initlist.append([fld.name()+':', init.shortrepr()])
                    else:
                        self.initlist.append([fld.name()+':',
                                              '<uninitialized>'])

    def copyfstateCB(self, gtkobj): # Button callback.
        menuitem = meshmenu.Copy_Field_State
        targetparam = menuitem.get_arg("target")

        targetparam.set(self.currentFullMeshName())
        if parameterwidgets.getParameters(targetparam,
                                          title="Select a target mesh"):
            menuitem.callWithDefaults(source=self.currentFullMeshName())

    def newSkeleton(self, skeletonname): # switchboard ("new who", "Skeleton")
        self.sensitize()

    def newMesh(self, meshname):        # switchboard ("new who", "Mesh")
        self.set_state(meshname)
        self.sensitize()
        
    def meshCB(self, meshpath):         # WhoWidget callback
        self.set_state(meshpath)
        self.sensitize()

    def removeMesh(self, path):    # switchboard ("remove who", "Mesh")
        self.update()
        self.sensitize()

    def reservationChanged(self, whoobj):
        # switchboard "made reservation", "cancelled reservation"
        if  self.currentFullMeshName() == whoobj.path():
            self.sensitize()

    def buttonpressCB(self, gtkwidget, gdkevent):
        if gdkevent.type == GDK._2BUTTON_PRESS:
            # A double-click generates a "select-row" signal, a
            # "button_press_event" with event type 2BUTTON_PRESS, and
            # then another "select-row" signal.  Although it would be
            # nice to call initbuttonCB() here directly upon receipt of
            # the double click, it doesn't work because initbuttonCB()
            # brings up a modal dialog box, and the next signal to be
            # processed is the second "select-row" for this GtkCList,
            # which isn't in the modal dialog.  That hangs X (at least
            # it hangs XFree86 on OS X).  It may be a bug in gtk+ or
            # X.  In any case, the solution is to delay raising the
            # modal dialog until the second "select-row" is issued.
            self.doubleclicked = 1

    def initSelectCB(self, gtkwidget, row, col, gdkevent):
        self.currentInit = row
        self.sensitize()
        if self.doubleclicked:
            self.doubleclicked = 0
            self.initbuttonCB()
            
    def initUnselectCB(self, *args):
        self.currentInit = None
        self.sensitize()

    def initbuttonCB(self, *args):      # gtk button callback
        menuitem = meshmenu.Initialize
        fieldname = self.initlist.get_text(self.currentInit, 0)[:-1]
        fld = allFields[fieldname]
        # We can't simply use
        # parameter.getParameters(menuitem.get_arg('initializer'))
        # here, because the menu items parameter accepts members of
        # the FieldInit base class, but we need to create a widget for
        # the appropriate derived class (ScalarFieldInit, et al).
        # It's the derived classes that are the registered classes.
        param = parameter.RegisteredParameter(
            'initializer', fieldinit.fieldInitDict[fld.classname()])
        param.value = self.currentMeshContext().get_initializer(fld)
        init = parameterwidgets.getParameterValues(
            param, title = 'Initialize field ' + fieldname)

        if init is not None:
            menuitem(mesh=self.currentFullMeshName(),
                     field=fld,
                     initializer=init[0])

    def initFldCB(self):
        self.updateInitializers()


#############

fp = FieldPage()
