# -*- python -*-
# $RCSfile: meshcswidget.py,v $
# $Revision: 1.21 $
# $Author: langer $
# $Date: 2005/03/02 21:25:07 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from SWIG.common import switchboard
from common.IO import selectionplaceholder
from common.IO import whoville
from common.IO.GUI import chooser
from common.IO.GUI import parameterwidgets
from common.IO.GUI import whowidget
from engine import mesh
from engine.IO import meshcsparams
from engine.IO import meshmenu

import gtk
import string


# Widget for MeshCrossSectionSet objects.  Just puts up a list of the
# things, and lets the user pick any number of them, or use the
# "selected" one, which has meaning in the context of a mesh object.
# Return value is a (possibly empty) list of strings, or
# selectionplaceholder.selectionPlaceHolder.

# This widget is currently used in the LayerEditor, so it may be
# assumed that a mesh widget that gives a nontrivial result will
# always be found.
class MeshCrossSectionSetParamWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param, scope):
        # Find the enclosing mesh widget.
        self.meshwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is mesh.meshes)

        self.gtk = gtk.GtkFrame()
        vbox = gtk.GtkVBox()
        self.gtk.add(vbox)

        meshname = self.meshwidget.get_value()
        try:
            self.meshobj = mesh.meshes[meshname]
        except KeyError:
            # no mesh!
            self.meshobj = None
            vbox.add(gtk.GtkLabel('No mesh!'))
        else:
            if issubclass(self.meshobj.__class__, whoville.WhoProxy):
                gfxwindow = scope.findData("gfxwindow")
                self.meshobj = mesh.resolve(gfxwindow)

            self.chooser = chooser.MultiListWidget()
            self.selected = gtk.GtkCheckButton("Selected")
            self.selected.connect("clicked", self.selectedCB)
            vbox.add(self.selected)
            vbox.add(self.chooser.gtk)
      
        parameterwidgets.ParameterWidget.__init__(self, self.gtk, scope)
        self.set_value(param.value)

        self.gtk.show_all()

    def selectedCB(self, gtkobj):
        if self.selected.get_active():
            self.set_value(selectionplaceholder.selectionPlaceHolder)
        else:
            self.chooser.clear()
            self.chooser.gtk.set_sensitive(1)
            lcsn=self.meshobj.selectedCSName()
            if lcsn:
                self.chooser.set_selection([lcsn])
        self.widgetChanged(1, interactive=1)

  
    def set_value(self, value):
        if self.meshobj is not None:
            self.chooser.update(self.meshobj.allCrossSectionNames())
            self.chooser.clear()
            if value==selectionplaceholder.selectionPlaceHolder:
                lcsn=self.meshobj.selectedCSName()
                if lcsn:
                    self.chooser.set_selection([lcsn])
                self.chooser.gtk.set_sensitive(0)
                self.selected.set_active(1)
            else:  # Value is list of names.
                self.chooser.gtk.set_sensitive(1)
                self.selected.set_active(0)
                self.chooser.set_selection(value)
            self.widgetChanged(1, interactive=0)
        else:
            self.widgetChanged(0, interactive=0)
          
    def get_value(self):
        if self.meshobj is not None:
            if self.selected.get_active():
                return selectionplaceholder.selectionPlaceHolder
            else:
                return self.chooser.get_value()

    def cleanUp(self):
        self.chooser = None
        parameterwidgets.ParameterWidget.cleanUp(self)

def _make_MCSSPWidget(self, scope):
    return MeshCrossSectionSetParamWidget(self, scope)

meshcsparams.MeshCrossSectionSetParameter.makeWidget = _make_MCSSPWidget

###################

# Widget for picking a single cross section from the set of cross
# sections already defined on a mesh.  Also includes buttons for
# defining and editing cross sections.

class MeshCrossSectionParamWidget(parameterwidgets.ParameterWidget):
    def __init__(self, param=None, scope=None):
        # Find the enclosing mesh widget.
        self.meshwidget = scope.findWidget(
            lambda x: isinstance(x, whowidget.WhoWidget)
            and x.whoclass is mesh.meshes)

        meshname = self.meshwidget.get_value()
        meshpath = string.split(meshname, ":")
        if meshpath[2]:
            self.meshobj = mesh.meshes[meshname]
            if issubclass(self.meshobj.__class__, whoville.WhoProxy):
                gfxwindow = scope.findData("gfxwindow")
                self.meshobj = mesh.resolve(gfxwindow)

        frame = gtk.GtkFrame()
        vbox = gtk.GtkVBox()
        frame.add(vbox)
        self.chooser = chooser.ChooserWidget([], callback=self.chooserCB)
        vbox.add(self.chooser.gtk)

        bbox = gtk.GtkHBox()
        bbox.set_spacing(1)
        vbox.add(bbox, padding=1)

        self.newbutton = gtk.GtkButton('New')
        self.newbutton.connect('clicked', self.newCB)
        bbox.add(self.newbutton)

        self.copybutton = gtk.GtkButton('Copy')
        self.copybutton.connect('clicked', self.copyCB)
        bbox.add(self.copybutton)

        self.editbutton = gtk.GtkButton('Edit')
        self.editbutton.connect('clicked', self.editCB)
        bbox.add(self.editbutton)

        self.renamebutton = gtk.GtkButton('Rename')
        self.renamebutton.connect('clicked', self.renameCB)
        bbox.add(self.renamebutton)

        self.deletebutton = gtk.GtkButton('Remove')
        self.deletebutton.connect('clicked', self.deleteCB)
        bbox.add(self.deletebutton)

        parameterwidgets.ParameterWidget.__init__(self, frame, scope)

        self.sbcbs = [
            switchboard.requestCallback(self.meshwidget, self.mesh_update),
            switchboard.requestCallback("cross sections changed",
                                        self.cs_update),
            switchboard.requestCallback(("cross section renamed", self.meshobj),
                                        self.renamedCS)
            ]
        
        if param:
            self.set_value(param.value)

        self.update(interactive=0)
        self.gtk.show_all()

    def mesh_update(self, interactive):
        self.update(interactive)
    def cs_update(self):
        self.update(interactive=0)
        
    def update(self, interactive):
        meshname = self.currentMeshName()
        meshpath = string.split(meshname, ':')
        if meshpath[2]:
            self.meshobj = mesh.meshes[meshname]
            self.chooser.update(self.meshobj.allCrossSectionNames())
            self.chooser.set_state(self.meshobj.selectedCSName())
            self.sensitize()
        else:
            self.meshobj = None
            self.chooser.update([])
            self.sensitize()
        # This update can affect the validity, of course.
        if self.chooser.get_value():
            self.widgetChanged(1, interactive)
        else:
            self.widgetChanged(0, interactive)
    def sensitize(self):
        meshok = self.meshobj is not None
        if meshok:
            ok = self.chooser.get_value() is not None
        else:
            ok = 0
        self.newbutton.set_sensitive(meshok)
        self.copybutton.set_sensitive(ok)
        self.editbutton.set_sensitive(ok)
        self.renamebutton.set_sensitive(ok)
        self.deletebutton.set_sensitive(ok)

    def currentMeshName(self):
        return self.meshwidget.get_value()

    def chooserCB(self, *args):
        self.widgetChanged(self.chooser.nChoices()>0, interactive=1)

    def newCB(self, *args):             # gtk callback
        menuitem = meshmenu.csmenu.New
        if parameterwidgets.getParameters(menuitem.get_arg("name"),
                                          menuitem.get_arg("cross_section"),
                                          title="New cross section",
                                          scope=self.scope):
            menuitem.callWithDefaults(mesh=self.meshwidget.get_value())

    def deleteCB(self, *args):          # gtk callback
        menuitem = meshmenu.csmenu.Remove
        menuitem.callWithDefaults(mesh=self.meshwidget.get_value(),
                                  name=self.chooser.get_value())

    def renameCB(self, *args):          # gtk callback
        menuitem = meshmenu.csmenu.Rename
        oldname = self.chooser.get_value()
        namearg = menuitem.get_arg('name')
        namearg.value = oldname
        if parameterwidgets.getParameters(namearg,
                                 title='Rename cross section "%s"' % oldname):
            menuitem.callWithDefaults(mesh=self.meshwidget.get_value(),
                                      cross_section=oldname)
    def renamedCS(self, oldname, newname): # sb ("cross section renamed", mesh)
        self.chooser.set_state(newname)

    def editCB(self, *args):            # gtk callback
        menuitem = meshmenu.csmenu.Edit
        csname = self.chooser.get_value()
        csparam = menuitem.get_arg('cross_section')
        csparam.value = self.meshobj.getCrossSection(csname)
        if parameterwidgets.getParameters(menuitem.get_arg('cross_section'),
                                          title="Edit cross section " + csname):
            menuitem.callWithDefaults(mesh=self.meshobj.path(), name=csname)
    def copyCB(self, *args):
        menuitem = meshmenu.csmenu.Copy
        csname = self.chooser.get_value()
        if parameterwidgets.getParameters(menuitem.get_arg('mesh'),
                                          menuitem.get_arg('name'),
                                          title="Copy cross section " + csname):
            menuitem.callWithDefaults(current=self.meshobj.path(),
                                      cross_section=csname)
            
    def set_value(self, value):
        self.chooser.set_state(value)
        if value:
            self.widgetChanged(1, interactive=0)
        else:
            self.widgetChanged(0, interactive=0)

    def get_value(self):
        return self.chooser.get_value()

    def cleanUp(self):
        self.chooser.destroy()
        for s in self.sbcbs:
            switchboard.removeCallback(s)
        parameterwidgets.ParameterWidget.cleanUp(self)
        

    
def _make_MCSPWidget(self, scope):
    return MeshCrossSectionParamWidget(self, scope)

meshcsparams.MeshCrossSectionParameter.makeWidget = _make_MCSPWidget
