# -*- python -*-
# $RCSfile: rationalshort.py,v $
# $Revision: 1.5 $
# $Author: langer $
# $Date: 2005/02/10 20:06:32 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov.

from common import registeredclass
from common.IO import parameter
from common.IO import xmlmenudump
from engine import rationalize
from engine import rationalizefunctions

Rationalizer = rationalize.Rationalizer
removeShortSide = rationalizefunctions.removeShortSide

class RemoveShortSide(Rationalizer):
    #
    #       C  _________
    #         |        / B   If a segment C-D is too short,
    #       D  \      /      it's better to merge C&D to form a triangle.
    #           \    /
    #            \  /
    #             \/      ===>     C ________
    #             A                  \      / B
    #                                 \    /
    #                                  \  /
    #                                   \/
    #                                    A
    #

    def __init__(self, ratio):
        self.ratio = ratio

    def findAndFix(self, skel, element):
        if element.nnodes() == 4:       # only applies to quads
            edgeLengths = element.getEdgeLengthsList()
            which = element.getShortestEdge()
            # See if the shortest edge is much shorter than the rest
            edgeLengths.sort()
            if edgeLengths[1]/edgeLengths[0] >= self.ratio:
                # Returns a list of ProvisionalChange objects
                return self.fix(skel, element, which)
        return []

    def fixAll(self, skel, element):
        if element.nnodes() == 4:
            which = element.getShortestEdge()
            return self.fix(skel, element, which)
        return []

    def fix(self, skel, element, which):
        return removeShortSide(skel, element, which)

registeredclass.Registration(
    'Remove Short Sides',
    Rationalizer,
    RemoveShortSide,
    gerund = 'removing short sides',
    ordering=0,
    params=[
    parameter.FloatParameter('ratio', value = 5.0,
                             tip = 'Maximum acceptable ratio of the lengths of the second shortest and the shortest sides.')
    ],
    tip = "Eliminate the shortest side of a quadrilateral.",
    discussion=xmlmenudump.loadFile('DISCUSSIONS/engine/reg-ration_short.xml'))
