# -*- python -*-
# $RCSfile: solver_.py,v $
# $Revision: 1.23 $
# $Author: redwing $
# $Date: 2005/03/01 16:31:13 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from common import cregisteredclass
from common import debug
from common.IO import parameter
from SWIG.common import devel
import SWIG.engine.preconditioner



def _apply(self, meshcontext):
    from SWIG.engine import solverstepper
    equilibrate = solverstepper.SolverStepper(
        self, meshcontext, solverstepper.SolverStepperIterationManager(1))
    equilibrate.precompute()
    equilibrate.iterate()
    equilibrate.short_postcompute()

    


LinearSolverPtr.apply = _apply



cregisteredclass.registerCClass(LinearSolverPtr)

LinearSolverPtr.tip = "Linear Solvers available in &oof2;."
LinearSolverPtr.discussion = "<para>TODO: Add discussion.</para>"


gmresSolver = cregisteredclass.Registration("GMRES", LinearSolverPtr, GMRESSolver, ordering=100,
                              params=[
    parameter.IntParameter("max_iterations", 1000,
                           tip="maximum number of iterations to perform"),
    parameter.IntParameter("krylov_dimension", 100,
                           tip="Something that can't easily be explained in a tooltip.  Making the Krylov dimension bigger will improve convergence but use more memory."),
    parameter.FloatParameter("tolerance", 1.e-8,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                                  SWIG.engine.preconditioner.PreconditionerWrapPtr,
                                  tip="Black magic for making the matrix more easily solvable")
    ],
                              tip="Generalized Minimal Residual method.  An iterative method for non-symmetric matrices."
                              )

cregisteredclass.Registration("BiCG", LinearSolverPtr, BiCGSolver, ordering=200,
                              params=[
    parameter.IntParameter("max_iterations", 1000,
                           tip="maximum number of iterations to perform"),
    parameter.FloatParameter("tolerance", 1.e-8,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                                  SWIG.engine.preconditioner.PreconditionerWrapPtr,
                                  tip="Black magic for making the matrix more easily solvable")
    ],
                              tip="Bi-Conjugate Gradient method.  An iterative method for non-symmetric matrices."
                              )

cregisteredclass.Registration("CG", LinearSolverPtr, CGSolver, ordering=300,
                              params=[
    parameter.IntParameter("max_iterations",  1000,
                           tip="maximum number of iterations to perform"),
    parameter.FloatParameter("tolerance", 1.e-8,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                        SWIG.engine.preconditioner.PreconditionerWrapPtr,
                        tip="Black magic for making the matrix more easily solvable")
    ],
                              tip="Conjugate Gradient method.  An iterative method  only for symmetric matrices.  OOF2 usually doesn't create symmetric stiffness matrices, so using CG isn't recommended unless you're sure you know what you're doing."
                              )

cregisteredclass.Registration("BiCGStab", LinearSolverPtr, BiCGStabSolver, ordering=400,
                              params=[
    parameter.IntParameter("max_iterations",  1000,
                           tip="maximum number of iterations to perform"),
    parameter.FloatParameter("tolerance", 1.e-8,
                             tip="largest acceptable relative error"),
    parameter.RegisteredParameter("preconditioner",
                        SWIG.engine.preconditioner.PreconditionerWrapPtr,
                        tip="Black magic for making the matrix more easily solvable")
    ],
                              tip="Bi-Conjugate Gradient method, Stabilized.  Another CG variant, applicable to non-symmetric matrices."
                              )
if devel.devel():
    cregisteredclass.Registration("Direct", LinearSolverPtr, DirectSolver,
                                  ordering=1000, params=[],
                                  tip="A non-iterative non-sparse matrix solver using LU decomposition.  Uses a lot of memory. Not recommended if the finite element mesh is large.")
    

from SWIG.engine.preconditioner import PreconditionerWrapPtr




